/*

this is a part of sap
copyright 2008 jezra lickter http://www.jezra.net

sap is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sap is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sap.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <glib.h>
#include <glib-object.h>
#include <gst/gst.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <float.h>
#include <math.h>


#define TYPE_URI_PLAYER (uri_player_get_type ())
#define URI_PLAYER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_URI_PLAYER, UriPlayer))
#define URI_PLAYER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_URI_PLAYER, UriPlayerClass))
#define IS_URI_PLAYER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_URI_PLAYER))
#define IS_URI_PLAYER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_URI_PLAYER))
#define URI_PLAYER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_URI_PLAYER, UriPlayerClass))

typedef struct _UriPlayer UriPlayer;
typedef struct _UriPlayerClass UriPlayerClass;
typedef struct _UriPlayerPrivate UriPlayerPrivate;

struct _UriPlayer {
	GObject parent_instance;
	UriPlayerPrivate * priv;
};

struct _UriPlayerClass {
	GObjectClass parent_class;
};

struct _UriPlayerPrivate {
	GstPipeline* pipeline;
	GstElement* src;
	GstElement* decoderbin;
	GstElement* audioconverter;
	GstElement* asink;
	GstElement* volume;
	GstQuery* duration_query;
	GstQuery* position_query;
	char* status_text;
	gint64 position;
	gint64 duration;
};



GType uri_player_get_type (void);
#define URI_PLAYER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_URI_PLAYER, UriPlayerPrivate))
enum  {
	URI_PLAYER_DUMMY_PROPERTY
};
static GstTagForeachFunc uri_player_handle_tags_delegate;
static gpointer uri_player_handle_tags_delegate_target;
static GstTagForeachFunc uri_player_handle_tags_delegate = NULL;
static gpointer uri_player_handle_tags_delegate_target = NULL;
void uri_player_handle_tags (UriPlayer* self, GstTagList* list, const char* tag);
static void _uri_player_handle_tags_gst_tag_foreach_func (GstTagList* list, const char* tag, gpointer self);
void uri_player_build_pipeline (UriPlayer* self);
UriPlayer* uri_player_new (const char* uri);
UriPlayer* uri_player_construct (GType object_type, const char* uri);
UriPlayer* uri_player_new (const char* uri);
void uri_player_change_uri (UriPlayer* self, const char* uri);
gboolean uri_player_new_decoded_pad (UriPlayer* self, GstPad* decodebin, gboolean arg1, void* data);
static void uri_player_bus_message (UriPlayer* self, GstMessage* message);
static void _lambda0_ (GstBus* bus, GstMessage* message, UriPlayer* self);
static void __lambda0__gst_bus_message (GstBus* _sender, GstMessage* message, gpointer self);
void uri_player_stop (UriPlayer* self);
void uri_player_play (UriPlayer* self);
void uri_player_pause (UriPlayer* self);
double uri_player_get_volume (UriPlayer* self);
void uri_player_set_volume (UriPlayer* self, double vol);
void uri_player_seek_forward (UriPlayer* self, gint percent);
void uri_player_seek_back (UriPlayer* self, gint percent);
void uri_player_increase_volume (UriPlayer* self, double val);
void uri_player_decrease_volume (UriPlayer* self, double val);
void uri_player_get_duration_info (UriPlayer* self);
static gpointer uri_player_parent_class = NULL;
static void uri_player_finalize (GObject* obj);


static void g_cclosure_user_marshal_VOID__INT64_INT64 (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);
static void g_cclosure_user_marshal_VOID__STRING_STRING (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);

static void _uri_player_handle_tags_gst_tag_foreach_func (GstTagList* list, const char* tag, gpointer self) {
	uri_player_handle_tags (self, list, tag);
}


/*start making the player*/
UriPlayer* uri_player_construct (GType object_type, const char* uri) {
	UriPlayer * self;
	GstTagForeachFunc _tmp0_;
	GstElement* _tmp1_;
	char* _tmp2_;
	GstQuery* _tmp3_;
	GstQuery* _tmp4_;
	g_return_val_if_fail (uri != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	uri_player_handle_tags_delegate = (_tmp0_ = _uri_player_handle_tags_gst_tag_foreach_func, uri_player_handle_tags_delegate_target = self, _tmp0_);
	self->priv->position = (gint64) 0;
	self->priv->duration = (gint64) 0;
	_tmp1_ = NULL;
	self->priv->src = (_tmp1_ = gst_element_make_from_uri (GST_URI_SRC, uri, "my_src"), (self->priv->src == NULL) ? NULL : (self->priv->src = (gst_object_unref (self->priv->src), NULL)), _tmp1_);
	_tmp2_ = NULL;
	self->priv->status_text = (_tmp2_ = g_strdup (""), self->priv->status_text = (g_free (self->priv->status_text), NULL), _tmp2_);
	fprintf (stdout, "uri set: %s\n", uri);
	uri_player_build_pipeline (self);
	_tmp3_ = NULL;
	self->priv->duration_query = (_tmp3_ = gst_query_new_duration (GST_FORMAT_TIME), (self->priv->duration_query == NULL) ? NULL : (self->priv->duration_query = (gst_query_unref (self->priv->duration_query), NULL)), _tmp3_);
	_tmp4_ = NULL;
	self->priv->position_query = (_tmp4_ = gst_query_new_position (GST_FORMAT_TIME), (self->priv->position_query == NULL) ? NULL : (self->priv->position_query = (gst_query_unref (self->priv->position_query), NULL)), _tmp4_);
	return self;
}


UriPlayer* uri_player_new (const char* uri) {
	return uri_player_construct (TYPE_URI_PLAYER, uri);
}


void uri_player_change_uri (UriPlayer* self, const char* uri) {
	GstElement* _tmp0_;
	GstElement* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (uri != NULL);
	/*TODO: there has to be a better way to change the uri
	perhaps by setting the uri property?*/
	gst_element_set_state ((GstElement*) self->priv->pipeline, GST_STATE_NULL);
	gst_bin_remove ((GstBin*) self->priv->pipeline, self->priv->src);
	_tmp0_ = NULL;
	self->priv->src = (_tmp0_ = gst_element_make_from_uri (GST_URI_SRC, uri, "my_src"), (self->priv->src == NULL) ? NULL : (self->priv->src = (gst_object_unref (self->priv->src), NULL)), _tmp0_);
	_tmp1_ = NULL;
	gst_bin_add ((GstBin*) self->priv->pipeline, (_tmp1_ = self->priv->src, (_tmp1_ == NULL) ? NULL : gst_object_ref (_tmp1_)));
	gst_element_link (self->priv->src, self->priv->decoderbin);
}


static void _lambda0_ (GstBus* bus, GstMessage* message, UriPlayer* self) {
	g_return_if_fail (bus != NULL);
	g_return_if_fail (message != NULL);
	uri_player_bus_message (self, message);
}


static void __lambda0__gst_bus_message (GstBus* _sender, GstMessage* message, gpointer self) {
	_lambda0_ (_sender, message, self);
}


void uri_player_build_pipeline (UriPlayer* self) {
	GstPipeline* _tmp0_;
	GstElement* _tmp1_;
	GstElement* _tmp2_;
	GstElement* _tmp3_;
	GstElement* _tmp4_;
	GstElement* _tmp5_;
	GstElement* _tmp6_;
	GstElement* _tmp7_;
	GstElement* _tmp8_;
	GstElement* _tmp9_;
	GstBus* bus;
	g_return_if_fail (self != NULL);
	/*make a gst pipe line to handle playing audio*/
	_tmp0_ = NULL;
	self->priv->pipeline = (_tmp0_ = GST_PIPELINE ((GstPipeline*) gst_pipeline_new ("pipeline")), (self->priv->pipeline == NULL) ? NULL : (self->priv->pipeline = (gst_object_unref (self->priv->pipeline), NULL)), _tmp0_);
	/*add our source*/
	_tmp1_ = NULL;
	gst_bin_add ((GstBin*) self->priv->pipeline, (_tmp1_ = self->priv->src, (_tmp1_ == NULL) ? NULL : gst_object_ref (_tmp1_)));
	/*making a decoder for all of the various audio type is a pain
	make a decoder bin and add it to the pipeline*/
	_tmp2_ = NULL;
	self->priv->decoderbin = (_tmp2_ = gst_element_factory_make ("decodebin", "decoder"), (self->priv->decoderbin == NULL) ? NULL : (self->priv->decoderbin = (gst_object_unref (self->priv->decoderbin), NULL)), _tmp2_);
	_tmp3_ = NULL;
	gst_bin_add ((GstBin*) self->priv->pipeline, (_tmp3_ = self->priv->decoderbin, (_tmp3_ == NULL) ? NULL : gst_object_ref (_tmp3_)));
	/*we need to let ourselves know when a new decoder is implemented, or do we?*/
	g_object_connect ((GObject*) self->priv->decoderbin, "swapped-object-signal::new-decoded-pad", uri_player_new_decoded_pad, self, NULL);
	/*we also need an audio converter*/
	_tmp4_ = NULL;
	self->priv->audioconverter = (_tmp4_ = gst_element_factory_make ("audioconvert", "converter"), (self->priv->audioconverter == NULL) ? NULL : (self->priv->audioconverter = (gst_object_unref (self->priv->audioconverter), NULL)), _tmp4_);
	_tmp5_ = NULL;
	gst_bin_add ((GstBin*) self->priv->pipeline, (_tmp5_ = self->priv->audioconverter, (_tmp5_ == NULL) ? NULL : gst_object_ref (_tmp5_)));
	/*create a volume element so we can change the volume*/
	_tmp6_ = NULL;
	self->priv->volume = (_tmp6_ = gst_element_factory_make ("volume", "vol"), (self->priv->volume == NULL) ? NULL : (self->priv->volume = (gst_object_unref (self->priv->volume), NULL)), _tmp6_);
	_tmp7_ = NULL;
	gst_bin_add ((GstBin*) self->priv->pipeline, (_tmp7_ = self->priv->volume, (_tmp7_ == NULL) ? NULL : gst_object_ref (_tmp7_)));
	_tmp8_ = NULL;
	self->priv->asink = (_tmp8_ = gst_element_factory_make ("autoaudiosink", "aoutput"), (self->priv->asink == NULL) ? NULL : (self->priv->asink = (gst_object_unref (self->priv->asink), NULL)), _tmp8_);
	_tmp9_ = NULL;
	gst_bin_add ((GstBin*) self->priv->pipeline, (_tmp9_ = self->priv->asink, (_tmp9_ == NULL) ? NULL : gst_object_ref (_tmp9_)));
	/*link everything up*/
	gst_element_link (self->priv->src, self->priv->decoderbin);
	gst_element_link (self->priv->audioconverter, self->priv->volume);
	gst_element_link (self->priv->volume, self->priv->asink);
	/*we need to receive signals from the pipelines bus*/
	bus = gst_pipeline_get_bus (self->priv->pipeline);
	/*make sure we are watching the signals on the bus*/
	gst_bus_add_signal_watch (bus);
	g_signal_connect (bus, "message", (GCallback) __lambda0__gst_bus_message, self);
	(bus == NULL) ? NULL : (bus = (gst_object_unref (bus), NULL));
}


/*stdout.printf("pipeline built\n");*/
void uri_player_handle_tags (UriPlayer* self, GstTagList* list, const char* tag) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (list != NULL);
	g_return_if_fail (tag != NULL);
	if (tag != NULL) {
		GQuark _tmp4_;
		const char* _tmp3_;
		_tmp3_ = NULL;
		static GQuark _tmp4__label0 = 0;
		static GQuark _tmp4__label1 = 0;
		static GQuark _tmp4__label2 = 0;
		static GQuark _tmp4__label3 = 0;
		static GQuark _tmp4__label4 = 0;
		static GQuark _tmp4__label5 = 0;
		static GQuark _tmp4__label6 = 0;
		static GQuark _tmp4__label7 = 0;
		_tmp3_ = tag;
		_tmp4_ = (NULL == _tmp3_) ? 0 : g_quark_from_string (_tmp3_);
		if ((((((_tmp4_ == ((0 != _tmp4__label0) ? _tmp4__label0 : (_tmp4__label0 = g_quark_from_static_string ("artist")))) || (_tmp4_ == ((0 != _tmp4__label1) ? _tmp4__label1 : (_tmp4__label1 = g_quark_from_static_string ("title"))))) || (_tmp4_ == ((0 != _tmp4__label2) ? _tmp4__label2 : (_tmp4__label2 = g_quark_from_static_string ("album"))))) || (_tmp4_ == ((0 != _tmp4__label3) ? _tmp4__label3 : (_tmp4__label3 = g_quark_from_static_string ("channel-mode"))))) || (_tmp4_ == ((0 != _tmp4__label4) ? _tmp4__label4 : (_tmp4__label4 = g_quark_from_static_string ("comment"))))) || (_tmp4_ == ((0 != _tmp4__label5) ? _tmp4__label5 : (_tmp4__label5 = g_quark_from_static_string ("audio-codec")))))
		do {
			char* val;
			char* _tmp2_;
			gboolean _tmp1_;
			char* _tmp0_;
			val = g_strdup ("Unknown");
			_tmp2_ = NULL;
			_tmp0_ = NULL;
			_tmp1_ = gst_tag_list_get_string (list, tag, &_tmp0_);
			val = (_tmp2_ = _tmp0_, val = (g_free (val), NULL), _tmp2_);
			_tmp1_;
			g_signal_emit_by_name (self, "tag-parsed", tag, val);
			val = (g_free (val), NULL);
			break;
		} while (0); else if ((_tmp4_ == ((0 != _tmp4__label6) ? _tmp4__label6 : (_tmp4__label6 = g_quark_from_static_string ("track-number")))) || (_tmp4_ == ((0 != _tmp4__label7) ? _tmp4__label7 : (_tmp4__label7 = g_quark_from_static_string ("bitrate")))))
		do {
			guint val;
			val = 0U;
			gst_tag_list_get_uint (list, tag, &val);
			break;
		} while (0); else
		do {
			break;
		} while (0);
	}
}


static void uri_player_bus_message (UriPlayer* self, GstMessage* message) {
	GstTagList* tag_list;
	GstMessageType t;
	g_return_if_fail (self != NULL);
	g_return_if_fail (message != NULL);
	tag_list = NULL;
	t = message->type;
	/*stdout.printf("message type:%s\n",Message.type_get_name(t));*/
	switch (t) {
		case GST_MESSAGE_ERROR:
		{
			gst_element_set_state ((GstElement*) self->priv->pipeline, GST_STATE_NULL);
			g_signal_emit_by_name (self, "finished-playing");
			break;
		}
		case GST_MESSAGE_EOS:
		{
			gst_element_set_state ((GstElement*) self->priv->pipeline, GST_STATE_NULL);
			g_signal_emit_by_name (self, "finished-playing");
			break;
		}
		case GST_MESSAGE_TAG:
		{
			GstTagList* _tmp1_;
			GstTagList* _tmp0_;
			_tmp1_ = NULL;
			_tmp0_ = NULL;
			gst_message_parse_tag (message, &_tmp0_);
			tag_list = (_tmp1_ = _tmp0_, (tag_list == NULL) ? NULL : (tag_list = (gst_tag_list_free (tag_list), NULL)), _tmp1_);
			gst_tag_list_foreach (tag_list, uri_player_handle_tags_delegate, uri_player_handle_tags_delegate_target);
			break;
		}
	}
	(tag_list == NULL) ? NULL : (tag_list = (gst_tag_list_free (tag_list), NULL));
}


gboolean uri_player_new_decoded_pad (UriPlayer* self, GstPad* decodebin, gboolean arg1, void* data) {
	GstPad* _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (decodebin != NULL, FALSE);
	/*stdout.printf("new decoded pad created\n");
	link the pad to the audioconverter*/
	_tmp0_ = NULL;
	gst_pad_link (decodebin, _tmp0_ = gst_element_get_static_pad (self->priv->audioconverter, "sink"));
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (gst_object_unref (_tmp0_), NULL));
	return TRUE;
}


void uri_player_stop (UriPlayer* self) {
	g_return_if_fail (self != NULL);
	gst_element_set_state ((GstElement*) self->priv->pipeline, GST_STATE_READY);
}


/*when the play command is sent*/
void uri_player_play (UriPlayer* self) {
	g_return_if_fail (self != NULL);
	gst_element_set_state ((GstElement*) self->priv->pipeline, GST_STATE_PLAYING);
	g_signal_emit_by_name (self, "player-status", "Playing");
}


/*stdout.printf("playing\n");*/
void uri_player_pause (UriPlayer* self) {
	GstState state;
	GstState pending;
	g_return_if_fail (self != NULL);
	state = 0;
	pending = 0;
	gst_element_get_state ((GstElement*) self->priv->pipeline, &state, &pending, (GstClockTime) 1);
	if (state == GST_STATE_PAUSED) {
		uri_player_play (self);
	} else {
		gst_element_set_state ((GstElement*) self->priv->pipeline, GST_STATE_PAUSED);
		g_signal_emit_by_name (self, "player-status", "Paused");
	}
}


double uri_player_get_volume (UriPlayer* self) {
	double vol;
	g_return_val_if_fail (self != NULL, 0.0);
	vol = 0.0;
	/*return the volume of the pipeline*/
	g_object_get ((GObject*) self->priv->volume, "volume", &vol, NULL);
	return vol;
}


void uri_player_set_volume (UriPlayer* self, double vol) {
	g_return_if_fail (self != NULL);
	/*set the volume of the pipeline*/
	g_object_set ((GObject*) self->priv->volume, "volume", vol, NULL);
	g_signal_emit_by_name (self, "volume-changed", vol);
}


void uri_player_seek_forward (UriPlayer* self, gint percent) {
	gint64 newloc;
	g_return_if_fail (self != NULL);
	newloc = 0LL;
	newloc = self->priv->position + ((self->priv->duration * percent) / 100);
	/*perform a simple seek*/
	gst_element_seek_simple (self->priv->audioconverter, GST_FORMAT_TIME, GST_SEEK_FLAG_KEY_UNIT | GST_SEEK_FLAG_FLUSH, newloc);
	uri_player_play (self);
}


void uri_player_seek_back (UriPlayer* self, gint percent) {
	gint64 newloc;
	g_return_if_fail (self != NULL);
	newloc = 0LL;
	if ((self->priv->position - ((self->priv->duration * percent) / 100)) < 0) {
		newloc = (gint64) 0;
	} else {
		newloc = self->priv->position - ((self->priv->duration * percent) / 100);
	}
	/*perform a simple seek*/
	gst_element_seek_simple (self->priv->audioconverter, GST_FORMAT_TIME, GST_SEEK_FLAG_KEY_UNIT | GST_SEEK_FLAG_FLUSH, newloc);
	uri_player_play (self);
}


void uri_player_increase_volume (UriPlayer* self, double val) {
	double newvol;
	double vol;
	g_return_if_fail (self != NULL);
	newvol = 0.0;
	vol = uri_player_get_volume (self);
	newvol = vol + val;
	if (vol >= 1) {
		newvol = (double) 1;
	}
	uri_player_set_volume (self, newvol);
}


void uri_player_decrease_volume (UriPlayer* self, double val) {
	double newvol;
	double vol;
	g_return_if_fail (self != NULL);
	newvol = 0.0;
	vol = uri_player_get_volume (self);
	if ((vol - val) < 0) {
		newvol = (double) 0;
	} else {
		newvol = vol - val;
	}
	uri_player_set_volume (self, newvol);
}


void uri_player_get_duration_info (UriPlayer* self) {
	gboolean duration_result;
	gboolean position_result;
	GstFormat format;
	gboolean _tmp0_;
	g_return_if_fail (self != NULL);
	duration_result = FALSE;
	position_result = FALSE;
	format = GST_FORMAT_TIME;
	duration_result = gst_element_query ((GstElement*) self->priv->pipeline, self->priv->duration_query);
	position_result = gst_element_query ((GstElement*) self->priv->pipeline, self->priv->position_query);
	_tmp0_ = FALSE;
	if (duration_result) {
		_tmp0_ = position_result;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		gst_query_parse_duration (self->priv->duration_query, &format, &self->priv->duration);
		gst_query_parse_position (self->priv->position_query, &format, &self->priv->position);
		g_signal_emit_by_name (self, "position-duration", self->priv->position, self->priv->duration);
	}
}


static void uri_player_class_init (UriPlayerClass * klass) {
	uri_player_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (UriPlayerPrivate));
	G_OBJECT_CLASS (klass)->finalize = uri_player_finalize;
	g_signal_new ("position_duration", TYPE_URI_PLAYER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__INT64_INT64, G_TYPE_NONE, 2, G_TYPE_INT64, G_TYPE_INT64);
	g_signal_new ("finished_playing", TYPE_URI_PLAYER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("tag_parsed", TYPE_URI_PLAYER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__STRING_STRING, G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_STRING);
	g_signal_new ("volume_changed", TYPE_URI_PLAYER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__DOUBLE, G_TYPE_NONE, 1, G_TYPE_DOUBLE);
	g_signal_new ("player_status", TYPE_URI_PLAYER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
}


static void uri_player_instance_init (UriPlayer * self) {
	self->priv = URI_PLAYER_GET_PRIVATE (self);
}


static void uri_player_finalize (GObject* obj) {
	UriPlayer * self;
	self = URI_PLAYER (obj);
	{
		gst_element_set_state ((GstElement*) self->priv->pipeline, GST_STATE_NULL);
	}
	(self->priv->pipeline == NULL) ? NULL : (self->priv->pipeline = (gst_object_unref (self->priv->pipeline), NULL));
	(self->priv->src == NULL) ? NULL : (self->priv->src = (gst_object_unref (self->priv->src), NULL));
	(self->priv->decoderbin == NULL) ? NULL : (self->priv->decoderbin = (gst_object_unref (self->priv->decoderbin), NULL));
	(self->priv->audioconverter == NULL) ? NULL : (self->priv->audioconverter = (gst_object_unref (self->priv->audioconverter), NULL));
	(self->priv->asink == NULL) ? NULL : (self->priv->asink = (gst_object_unref (self->priv->asink), NULL));
	(self->priv->volume == NULL) ? NULL : (self->priv->volume = (gst_object_unref (self->priv->volume), NULL));
	(self->priv->duration_query == NULL) ? NULL : (self->priv->duration_query = (gst_query_unref (self->priv->duration_query), NULL));
	(self->priv->position_query == NULL) ? NULL : (self->priv->position_query = (gst_query_unref (self->priv->position_query), NULL));
	self->priv->status_text = (g_free (self->priv->status_text), NULL);
	G_OBJECT_CLASS (uri_player_parent_class)->finalize (obj);
}


GType uri_player_get_type (void) {
	static GType uri_player_type_id = 0;
	if (uri_player_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (UriPlayerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) uri_player_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (UriPlayer), 0, (GInstanceInitFunc) uri_player_instance_init, NULL };
		uri_player_type_id = g_type_register_static (G_TYPE_OBJECT, "UriPlayer", &g_define_type_info, 0);
	}
	return uri_player_type_id;
}



static void g_cclosure_user_marshal_VOID__INT64_INT64 (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__INT64_INT64) (gpointer data1, gint64 arg_1, gint64 arg_2, gpointer data2);
	register GMarshalFunc_VOID__INT64_INT64 callback;
	register GCClosure * cc;
	register gpointer data1, data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 3);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__INT64_INT64) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_int64 (param_values + 1), g_value_get_int64 (param_values + 2), data2);
}


static void g_cclosure_user_marshal_VOID__STRING_STRING (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__STRING_STRING) (gpointer data1, const char* arg_1, const char* arg_2, gpointer data2);
	register GMarshalFunc_VOID__STRING_STRING callback;
	register GCClosure * cc;
	register gpointer data1, data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 3);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__STRING_STRING) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_string (param_values + 1), g_value_get_string (param_values + 2), data2);
}



