/*

this is a part of sap
copyright 2008 jezra lickter http://www.jezra.net

sap is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sap is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sap.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <glib.h>
#include <glib-object.h>
#include <stdio.h>
#include <curses.h>
#include <stdlib.h>
#include <string.h>


#define TYPE_WIN (win_get_type ())
#define WIN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_WIN, Win))
#define WIN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_WIN, WinClass))
#define IS_WIN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_WIN))
#define IS_WIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_WIN))
#define WIN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_WIN, WinClass))

typedef struct _Win Win;
typedef struct _WinClass WinClass;
typedef struct _WinPrivate WinPrivate;

struct _Win {
	GObject parent_instance;
	WinPrivate * priv;
};

struct _WinClass {
	GObjectClass parent_class;
};

struct _WinPrivate {
	gint c;
	gboolean looping;
};



GType win_get_type (void);
#define WIN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_WIN, WinPrivate))
enum  {
	WIN_DUMMY_PROPERTY
};
static void win_quit_app (Win* self);
void* win_ui_thread (Win* self);
static void* _win_ui_thread_gthread_func (gpointer self);
gboolean win_run (Win* self);
void win_clear_screen (Win* self);
void win_write_line (Win* self, gint line, const char* str);
void win_set_artist (Win* self, const char* artist);
void win_set_title (Win* self, const char* title);
void win_set_album (Win* self, const char* album);
void win_set_status (Win* self, const char* str);
void win_set_played_percent (Win* self, gint per);
Win* win_new (void);
Win* win_construct (GType object_type);
Win* win_new (void);
static GObject * win_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer win_parent_class = NULL;
static void win_finalize (GObject* obj);



static void win_quit_app (Win* self) {
	g_return_if_fail (self != NULL);
	self->priv->looping = FALSE;
	/*pass a quit signal*/
	g_signal_emit_by_name (self, "quit");
}


static void* _win_ui_thread_gthread_func (gpointer self) {
	return win_ui_thread (self);
}


gboolean win_run (Win* self) {
	GError * _inner_error_;
	g_return_val_if_fail (self != NULL, FALSE);
	_inner_error_ = NULL;
	{
		g_thread_create (_win_ui_thread_gthread_func, self, FALSE, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_THREAD_ERROR) {
				goto __catch0_g_thread_error;
			}
			goto __finally0;
		}
		return TRUE;
	}
	goto __finally0;
	__catch0_g_thread_error:
	{
		GError * ex;
		ex = _inner_error_;
		_inner_error_ = NULL;
		{
			gboolean _tmp1_;
			fprintf (stderr, "Failed to create the UI thread");
			return (_tmp1_ = FALSE, (ex == NULL) ? NULL : (ex = (g_error_free (ex), NULL)), _tmp1_);
		}
	}
	__finally0:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
}


void* win_ui_thread (Win* self) {
	g_return_val_if_fail (self != NULL, NULL);
	while (self->priv->looping) {
		self->priv->c = getch ();
		switch (self->priv->c) {
			case 'q':
			{
				win_quit_app (self);
				break;
			}
			case KEY_LEFT:
			case 'h':
			{
				g_signal_emit_by_name (self, "seek-back");
				break;
			}
			case KEY_RIGHT:
			case 'l':
			{
				g_signal_emit_by_name (self, "seek-forward");
				break;
			}
			case KEY_UP:
			case 'k':
			{
				g_signal_emit_by_name (self, "change-volume-up");
				break;
			}
			case KEY_DOWN:
			case 'j':
			{
				g_signal_emit_by_name (self, "change-volume-down");
				break;
			}
			case ' ':
			{
				g_signal_emit_by_name (self, "pause");
				break;
			}
			case 'b':
			case 68:
			{
				g_signal_emit_by_name (self, "previous-track");
				break;
			}
			case 'n':
			case 67:
			{
				g_signal_emit_by_name (self, "next-track");
				break;
			}
			default:
			{
				break;
			}
		}
	}
	/*do nothing*/
	return NULL;
}


void win_clear_screen (Win* self) {
	g_return_if_fail (self != NULL);
	/*move the drawing cursor to the first char of the 5th line
	from the bottom*/
	move (4, 0);
	/*clear the screen from cursor to the bottom of the screen*/
	clrtobot ();
}


void win_set_artist (Win* self, const char* artist) {
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (artist != NULL);
	_tmp0_ = NULL;
	win_write_line (self, LINES - 4, _tmp0_ = g_strdup_printf ("Artist: %s", artist));
	_tmp0_ = (g_free (_tmp0_), NULL);
}


void win_set_title (Win* self, const char* title) {
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (title != NULL);
	_tmp0_ = NULL;
	win_write_line (self, LINES - 5, _tmp0_ = g_strdup_printf ("Title: %s", title));
	_tmp0_ = (g_free (_tmp0_), NULL);
}


void win_set_album (Win* self, const char* album) {
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (album != NULL);
	_tmp0_ = NULL;
	win_write_line (self, LINES - 3, _tmp0_ = g_strdup_printf ("Album: %s", album));
	_tmp0_ = (g_free (_tmp0_), NULL);
}


void win_set_status (Win* self, const char* str) {
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (str != NULL);
	_tmp0_ = NULL;
	win_write_line (self, LINES - 2, _tmp0_ = g_strdup_printf ("%s", str));
	_tmp0_ = (g_free (_tmp0_), NULL);
}


void win_write_line (Win* self, gint line, const char* str) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (str != NULL);
	move (line, 0);
	clrtoeol ();
	addstr (str);
	/*update the screen*/
	refresh ();
}


void win_set_played_percent (Win* self, gint per) {
	gint char_num;
	gint pad_num;
	g_return_if_fail (self != NULL);
	/*how many chars is this?*/
	char_num = ((gint) ((COLS * per) / 100)) - 1;
	pad_num = (COLS - char_num) - 3;
	/*clear the bottom line
	move to the bottom line*/
	move (LINES - 1, 0);
	clrtoeol ();
	/*add the chars*/
	addch ((gulong) '[');
	{
		gint i;
		i = 0;
		for (; i < char_num; i++) {
			addch ((gulong) '=');
		}
	}
	addch ((gulong) '|');
	{
		gint i;
		i = 0;
		for (; i < pad_num; i++) {
			addch ((gulong) '-');
		}
	}
	addch ((gulong) ']');
}


Win* win_construct (GType object_type) {
	Win * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


Win* win_new (void) {
	return win_construct (TYPE_WIN);
}


static GObject * win_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	WinClass * klass;
	GObjectClass * parent_class;
	Win * self;
	klass = WIN_CLASS (g_type_class_peek (TYPE_WIN));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = WIN (obj);
	{
		self->priv->looping = TRUE;
		/*create the screen*/
		initscr ();
		/*Curses.stdscr is the standard screen
		hide the cursor*/
		curs_set (0);
		/*set Curses to not echo keypresses*/
		noecho ();
		/*do not wait for a "return" to process user input*/
		cbreak ();
		/*we need to get the keypad presses*/
		keypad (stdscr, TRUE);
	}
	return obj;
}


static void win_class_init (WinClass * klass) {
	win_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (WinPrivate));
	G_OBJECT_CLASS (klass)->constructor = win_constructor;
	G_OBJECT_CLASS (klass)->finalize = win_finalize;
	g_signal_new ("seek_forward", TYPE_WIN, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("seek_back", TYPE_WIN, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("pause", TYPE_WIN, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("quit", TYPE_WIN, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("change_volume_up", TYPE_WIN, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("change_volume_down", TYPE_WIN, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("next_track", TYPE_WIN, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("previous_track", TYPE_WIN, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void win_instance_init (Win * self) {
	self->priv = WIN_GET_PRIVATE (self);
}


static void win_finalize (GObject* obj) {
	Win * self;
	self = WIN (obj);
	{
		/*do a little clean up*/
		move (0, 0);
		deleteln ();
		refresh ();
		nocbreak ();
		echo ();
		endwin ();
	}
	G_OBJECT_CLASS (win_parent_class)->finalize (obj);
}


GType win_get_type (void) {
	static GType win_type_id = 0;
	if (win_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (WinClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) win_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Win), 0, (GInstanceInitFunc) win_instance_init, NULL };
		win_type_id = g_type_register_static (G_TYPE_OBJECT, "Win", &g_define_type_info, 0);
	}
	return win_type_id;
}




