/*

this is a part of sap
copyright 2008 jezra lickter http://www.jezra.net

sap is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sap is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sap.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <glib/gstdio.h>
#include <stdio.h>
#include <gst/gst.h>


#define TYPE_CONTROLLER (controller_get_type ())
#define CONTROLLER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONTROLLER, Controller))
#define CONTROLLER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONTROLLER, ControllerClass))
#define IS_CONTROLLER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONTROLLER))
#define IS_CONTROLLER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONTROLLER))
#define CONTROLLER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONTROLLER, ControllerClass))

typedef struct _Controller Controller;
typedef struct _ControllerClass ControllerClass;
typedef struct _ControllerPrivate ControllerPrivate;

#define TYPE_URI_PLAYER (uri_player_get_type ())
#define URI_PLAYER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_URI_PLAYER, UriPlayer))
#define URI_PLAYER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_URI_PLAYER, UriPlayerClass))
#define IS_URI_PLAYER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_URI_PLAYER))
#define IS_URI_PLAYER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_URI_PLAYER))
#define URI_PLAYER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_URI_PLAYER, UriPlayerClass))

typedef struct _UriPlayer UriPlayer;
typedef struct _UriPlayerClass UriPlayerClass;

#define TYPE_WIN (win_get_type ())
#define WIN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_WIN, Win))
#define WIN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_WIN, WinClass))
#define IS_WIN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_WIN))
#define IS_WIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_WIN))
#define WIN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_WIN, WinClass))

typedef struct _Win Win;
typedef struct _WinClass WinClass;

/*Define the Controller class*/
struct _Controller {
	GObject parent_instance;
	ControllerPrivate * priv;
	char* file_to_open;
};

struct _ControllerClass {
	GObjectClass parent_class;
};

struct _ControllerPrivate {
	UriPlayer* myPlayer;
	Win* myWin;
	char** file_array;
	gint file_array_length1;
	gint file_array_size;
	gint file_array_index;
	char* version;
	char* summary;
	char* description;
	char* status_text;
	GMainLoop* loop;
	char* ap_play_time;
	char* ap_total_time;
	double ap_volume;
	gint ap_percent_played;
	char* ap_state;
	GRegex* web_regex;
	GOptionContext* optContext;
};



GType controller_get_type (void);
GType uri_player_get_type (void);
GType win_get_type (void);
#define CONTROLLER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_CONTROLLER, ControllerPrivate))
enum  {
	CONTROLLER_DUMMY_PROPERTY
};
extern char** controller__uri;
char** controller__uri = NULL;
extern gboolean controller__list;
gboolean controller__list = FALSE;
extern gboolean controller__random;
gboolean controller__random = FALSE;
extern gboolean controller__continuous;
gboolean controller__continuous = FALSE;
void controller_ParseArgs (Controller* self, char*** args, int* args_length1, GError** error);
static Controller* controller_new (void);
static Controller* controller_construct (GType object_type);
static Controller* controller_new (void);
static void controller_print_help (Controller* self, const char* message);
static void controller_process_file_uri (Controller* self, const char* file_uri_path);
static void controller_process_file_uri_list (Controller* self, const char* uri_list);
static void _vala_array_add1 (char*** array, int* length, int* size, char* value);
static void controller_start_playing (Controller* self);
void controller_run (Controller* self);
UriPlayer* uri_player_new (const char* uri);
UriPlayer* uri_player_construct (GType object_type, const char* uri);
double uri_player_get_volume (UriPlayer* self);
static void controller_finished_playing (Controller* self);
static void _lambda1_ (UriPlayer* player, Controller* self);
static void __lambda1__uri_player_finished_playing (UriPlayer* _sender, gpointer self);
static void controller_process_position_duration (Controller* self, gint64 position, gint64 duration);
static void _lambda2_ (UriPlayer* player, gint64 p, gint64 d, Controller* self);
static void __lambda2__uri_player_position_duration (UriPlayer* _sender, gint64 position, gint64 duration, gpointer self);
static void controller_process_tag (Controller* self, const char* tag, const char* val);
static void _lambda3_ (UriPlayer* player, const char* tag, const char* val, Controller* self);
static void __lambda3__uri_player_tag_parsed (UriPlayer* _sender, const char* key, const char* val, gpointer self);
static void _lambda4_ (UriPlayer* player, double vol, Controller* self);
static void __lambda4__uri_player_volume_changed (UriPlayer* _sender, double vol, gpointer self);
static void _lambda5_ (UriPlayer* player, const char* stat, Controller* self);
static void __lambda5__uri_player_player_status (UriPlayer* _sender, const char* stat, gpointer self);
Win* win_new (void);
Win* win_construct (GType object_type);
static void controller_seek_forward (Controller* self);
static void _lambda6_ (Win* obj, Controller* self);
static void __lambda6__win_seek_forward (Win* _sender, gpointer self);
static void controller_seek_back (Controller* self);
static void _lambda7_ (Win* obj, Controller* self);
static void __lambda7__win_seek_back (Win* _sender, gpointer self);
static void controller_volume_up (Controller* self);
static void _lambda8_ (Win* obj, Controller* self);
static void __lambda8__win_change_volume_up (Win* _sender, gpointer self);
static void controller_volume_down (Controller* self);
static void _lambda9_ (Win* obj, Controller* self);
static void __lambda9__win_change_volume_down (Win* _sender, gpointer self);
static void controller_pause (Controller* self);
static void _lambda10_ (Win* obj, Controller* self);
static void __lambda10__win_pause (Win* _sender, gpointer self);
static void controller_quit_running (Controller* self);
static void _lambda11_ (Win* obj, Controller* self);
static void __lambda11__win_quit (Win* _sender, gpointer self);
static void controller_next_track (Controller* self);
static void _lambda12_ (Win* obj, Controller* self);
static void __lambda12__win_next_track (Win* _sender, gpointer self);
static void controller_previous_track (Controller* self);
static void _lambda13_ (Win* obj, Controller* self);
static void __lambda13__win_previous_track (Win* _sender, gpointer self);
gboolean win_run (Win* self);
static gboolean controller_get_player_duration_info (Controller* self);
static gboolean _controller_get_player_duration_info_gsource_func (gpointer self);
static void controller_play_audio (Controller* self);
void uri_player_play (UriPlayer* self);
void uri_player_change_uri (UriPlayer* self, const char* uri);
void win_clear_screen (Win* self);
void uri_player_get_duration_info (UriPlayer* self);
void win_set_artist (Win* self, const char* artist);
void win_set_title (Win* self, const char* title);
void win_set_album (Win* self, const char* album);
static char* controller_get_human_time (Controller* self, gint64 ns);
void win_set_status (Win* self, const char* str);
static void controller_update_status (Controller* self);
void win_set_played_percent (Win* self, gint per);
void uri_player_seek_forward (UriPlayer* self, gint percent);
void uri_player_seek_back (UriPlayer* self, gint percent);
void uri_player_increase_volume (UriPlayer* self, double val);
void uri_player_decrease_volume (UriPlayer* self, double val);
void uri_player_pause (UriPlayer* self);
static gpointer controller_parent_class = NULL;
static void controller_finalize (GObject* obj);
static char** _vala_array_dup1 (char** self, int length);
void _main (char** args, int args_length1);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);

static const GOptionEntry CONTROLLER_options[] = {{"list", 'l', (gint) G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_NONE, &controller__list, "file is a list", NULL}, {"random", 'r', (gint) G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_NONE, &controller__random, "play files randomly ( used with a list )", NULL}, {"continuous", 'c', (gint) G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_NONE, &controller__continuous, "play files continuously ( used with a list )", NULL}, {"", 'f', 0, G_OPTION_ARG_STRING_ARRAY, &controller__uri, "file(s) to play", "file(s)"}, {NULL}};


void controller_ParseArgs (Controller* self, char*** args, int* args_length1, GError** error) {
	GError * _inner_error_;
	GOptionContext* _tmp0_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	_tmp0_ = NULL;
	self->priv->optContext = (_tmp0_ = g_option_context_new (""), (self->priv->optContext == NULL) ? NULL : (self->priv->optContext = (g_option_context_free (self->priv->optContext), NULL)), _tmp0_);
	g_option_context_set_help_enabled (self->priv->optContext, TRUE);
	g_option_context_set_summary (self->priv->optContext, self->priv->summary);
	g_option_context_set_description (self->priv->optContext, self->priv->description);
	g_option_context_add_main_entries (self->priv->optContext, CONTROLLER_options, NULL);
	g_option_context_set_ignore_unknown_options (self->priv->optContext, TRUE);
	g_option_context_parse (self->priv->optContext, &(*args_length1), &(*args), &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_OPTION_ERROR) {
			g_propagate_error (error, _inner_error_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
}


/*init the controller*/
static Controller* controller_construct (GType object_type) {
	GError * _inner_error_;
	Controller * self;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	char* _tmp3_;
	char* _tmp4_;
	char* _tmp5_;
	GRegex* _tmp6_;
	_inner_error_ = NULL;
	self = g_object_newv (object_type, 0, NULL);
	_tmp0_ = NULL;
	self->priv->version = (_tmp0_ = g_strdup ("0.4.4"), self->priv->version = (g_free (self->priv->version), NULL), _tmp0_);
	_tmp1_ = NULL;
	self->priv->summary = (_tmp1_ = g_strdup_printf ("sap %s\nsap is a Simple Audio Player", self->priv->version), self->priv->summary = (g_free (self->priv->summary), NULL), _tmp1_);
	_tmp2_ = NULL;
	self->priv->description = (_tmp2_ = g_strdup ("-examples-\nsap file1 file2 file3\nthis will play file1,file2 and file3 in order and then quit\n\nsap -c file1 file2 file3\nwill play file1,file2 and file3 in order and repeat continuously\n\nsap -r file1 file2 file3\nwill play file1,file2 and file3 randomly and continuously\n\nsap -l playlist.txt\nwill play files found in a playlist\n\nsap -r -l playlist.txt\nwill randomly play files found in a playlist\n\n\n-controls-\nThe following keypresses will control sap as it is playing:\nSpace Bar : pause\nq : quit\nDown Arrow : volume down\nUp Arrow : volume up\nLeft Arrow : seek back\nRight Arrow : seek forward\nb : play previous file in list\nn : play next file in list\n\nFor VI(M) users, the \"h,j,k,l\" keys behave in a similar fashion\nto the Arrow keys.If you encounter any bugs, please report them at https://bugs.launchpad.net/sap+"), self->priv->description = (g_free (self->priv->description), NULL), _tmp2_);
	_tmp3_ = NULL;
	self->priv->ap_play_time = (_tmp3_ = g_strdup (""), self->priv->ap_play_time = (g_free (self->priv->ap_play_time), NULL), _tmp3_);
	_tmp4_ = NULL;
	self->priv->ap_total_time = (_tmp4_ = g_strdup (""), self->priv->ap_total_time = (g_free (self->priv->ap_total_time), NULL), _tmp4_);
	self->priv->ap_volume = (double) 0;
	_tmp5_ = NULL;
	self->priv->status_text = (_tmp5_ = g_strdup (""), self->priv->status_text = (g_free (self->priv->status_text), NULL), _tmp5_);
	_tmp6_ = NULL;
	self->priv->web_regex = (_tmp6_ = g_regex_new ("^http://.{0,100}", G_REGEX_CASELESS, 0, &_inner_error_), (self->priv->web_regex == NULL) ? NULL : (self->priv->web_regex = (g_regex_unref (self->priv->web_regex), NULL)), _tmp6_);
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	return self;
}


static Controller* controller_new (void) {
	return controller_construct (TYPE_CONTROLLER);
}


static void controller_process_file_uri_list (Controller* self, const char* uri_list) {
	GError * _inner_error_;
	char* file_contents;
	g_return_if_fail (self != NULL);
	g_return_if_fail (uri_list != NULL);
	_inner_error_ = NULL;
	file_contents = g_strdup ("");
	/*does the list exist?*/
	if (g_file_test (uri_list, G_FILE_TEST_EXISTS)) {
		char** _tmp4_;
		gint file_uris_size;
		gint file_uris_length1;
		char** _tmp3_;
		char** file_uris;
		{
			char* _tmp2_;
			gboolean _tmp1_;
			char* _tmp0_;
			_tmp2_ = NULL;
			_tmp0_ = NULL;
			_tmp1_ = g_file_get_contents (uri_list, &_tmp0_, NULL, &_inner_error_);
			file_contents = (_tmp2_ = _tmp0_, file_contents = (g_free (file_contents), NULL), _tmp2_);
			_tmp1_;
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == G_FILE_ERROR) {
					goto __catch1_g_file_error;
				}
				goto __finally1;
			}
		}
		goto __finally1;
		__catch1_g_file_error:
		{
			GError * err;
			err = _inner_error_;
			_inner_error_ = NULL;
			{
				controller_print_help (self, err->message);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
				file_contents = (g_free (file_contents), NULL);
				return;
			}
		}
		__finally1:
		if (_inner_error_ != NULL) {
			file_contents = (g_free (file_contents), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
		_tmp4_ = NULL;
		_tmp3_ = NULL;
		file_uris = (_tmp4_ = _tmp3_ = g_strsplit (file_contents, "\n", 0), file_uris_length1 = _vala_array_length (_tmp3_), file_uris_size = file_uris_length1, _tmp4_);
		{
			char** uri_collection;
			int uri_collection_length1;
			int uri_it;
			uri_collection = file_uris;
			uri_collection_length1 = file_uris_length1;
			for (uri_it = 0; uri_it < file_uris_length1; uri_it = uri_it + 1) {
				const char* _tmp5_;
				char* uri;
				_tmp5_ = NULL;
				uri = (_tmp5_ = uri_collection[uri_it], (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_));
				{
					controller_process_file_uri (self, uri);
					uri = (g_free (uri), NULL);
				}
			}
		}
		file_uris = (_vala_array_free (file_uris, file_uris_length1, (GDestroyNotify) g_free), NULL);
	} else {
		char* _tmp6_;
		_tmp6_ = NULL;
		controller_print_help (self, _tmp6_ = g_strdup_printf ("%s does not exist\n", uri_list));
		_tmp6_ = (g_free (_tmp6_), NULL);
	}
	file_contents = (g_free (file_contents), NULL);
}


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


static void _vala_array_add1 (char*** array, int* length, int* size, char* value) {
	if ((*length) == (*size)) {
		*size = (*size) ? (2 * (*size)) : 4;
		*array = g_renew (char*, *array, (*size) + 1);
	}
	(*array)[(*length)++] = value;
	(*array)[*length] = NULL;
}


static void controller_process_file_uri (Controller* self, const char* file_uri_path) {
	char* file_uri;
	g_return_if_fail (self != NULL);
	g_return_if_fail (file_uri_path != NULL);
	if (string_get_length (file_uri_path) == 0) {
		return;
	}
	file_uri = NULL;
	if (g_file_test (file_uri_path, G_FILE_TEST_EXISTS)) {
		/*is this an absolute path?*/
		if (g_path_is_absolute (file_uri_path)) {
			char* _tmp0_;
			_tmp0_ = NULL;
			file_uri = (_tmp0_ = g_strconcat ("file://", file_uri_path, NULL), file_uri = (g_free (file_uri), NULL), _tmp0_);
		} else {
			char* _tmp3_;
			char* _tmp2_;
			char* _tmp1_;
			/*if the path is not an absolute file, then create an abs file path*/
			_tmp3_ = NULL;
			_tmp2_ = NULL;
			_tmp1_ = NULL;
			file_uri = (_tmp3_ = g_strconcat ("file://", _tmp2_ = g_build_filename (_tmp1_ = g_get_current_dir (), file_uri_path, NULL), NULL), file_uri = (g_free (file_uri), NULL), _tmp3_);
			_tmp2_ = (g_free (_tmp2_), NULL);
			_tmp1_ = (g_free (_tmp1_), NULL);
		}
	} else {
		if (g_regex_match (self->priv->web_regex, file_uri_path, 0, NULL)) {
			char* _tmp5_;
			const char* _tmp4_;
			_tmp5_ = NULL;
			_tmp4_ = NULL;
			file_uri = (_tmp5_ = (_tmp4_ = file_uri_path, (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), file_uri = (g_free (file_uri), NULL), _tmp5_);
		} else {
		}
	}
	/*the file is not good*/
	if (file_uri != NULL) {
		const char* _tmp6_;
		/*add the uri to the file array*/
		_tmp6_ = NULL;
		_vala_array_add1 (&self->priv->file_array, &self->priv->file_array_length1, &self->priv->file_array_size, (_tmp6_ = file_uri, (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_)));
	}
	file_uri = (g_free (file_uri), NULL);
}


/* when happens when the controller runs*/
void controller_run (Controller* self) {
	g_return_if_fail (self != NULL);
	/*is there a _uri to process?*/
	if (controller__uri != NULL) {
		/*is the file path a list?*/
		if (controller__list) {
			/*we need to parse the list*/
			controller_process_file_uri_list (self, controller__uri[0]);
			if (self->priv->file_array_length1 == 0) {
				char* _tmp0_;
				_tmp0_ = NULL;
				controller_print_help (self, _tmp0_ = g_strdup_printf ("%s is not a list of files\n", controller__uri[0]));
				_tmp0_ = (g_free (_tmp0_), NULL);
			}
		} else {
			/*process the _uri array*/
			{
				char** uri_collection;
				int uri_collection_length1;
				int uri_it;
				uri_collection = controller__uri;
				uri_collection_length1 = _vala_array_length (controller__uri);
				for (uri_it = 0; uri_it < _vala_array_length (controller__uri); uri_it = uri_it + 1) {
					const char* _tmp1_;
					char* uri;
					_tmp1_ = NULL;
					uri = (_tmp1_ = uri_collection[uri_it], (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
					{
						controller_process_file_uri (self, uri);
						uri = (g_free (uri), NULL);
					}
				}
			}
		}
		/*set the file_array_index to 0*/
		self->priv->file_array_index = 0;
		if (self->priv->file_array_length1 > 0) {
			controller_start_playing (self);
		}
	} else {
		/*there is no file, print the help*/
		controller_print_help (self, "");
	}
}


static void _lambda1_ (UriPlayer* player, Controller* self) {
	g_return_if_fail (player != NULL);
	controller_finished_playing (self);
}


static void __lambda1__uri_player_finished_playing (UriPlayer* _sender, gpointer self) {
	_lambda1_ (_sender, self);
}


static void _lambda2_ (UriPlayer* player, gint64 p, gint64 d, Controller* self) {
	g_return_if_fail (player != NULL);
	controller_process_position_duration (self, p, d);
}


static void __lambda2__uri_player_position_duration (UriPlayer* _sender, gint64 position, gint64 duration, gpointer self) {
	_lambda2_ (_sender, position, duration, self);
}


static void _lambda3_ (UriPlayer* player, const char* tag, const char* val, Controller* self) {
	g_return_if_fail (player != NULL);
	g_return_if_fail (tag != NULL);
	g_return_if_fail (val != NULL);
	controller_process_tag (self, tag, val);
}


static void __lambda3__uri_player_tag_parsed (UriPlayer* _sender, const char* key, const char* val, gpointer self) {
	_lambda3_ (_sender, key, val, self);
}


static void _lambda4_ (UriPlayer* player, double vol, Controller* self) {
	g_return_if_fail (player != NULL);
	self->priv->ap_volume = vol;
}


static void __lambda4__uri_player_volume_changed (UriPlayer* _sender, double vol, gpointer self) {
	_lambda4_ (_sender, vol, self);
}


static void _lambda5_ (UriPlayer* player, const char* stat, Controller* self) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (player != NULL);
	g_return_if_fail (stat != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->ap_state = (_tmp1_ = (_tmp0_ = stat, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->ap_state = (g_free (self->priv->ap_state), NULL), _tmp1_);
}


static void __lambda5__uri_player_player_status (UriPlayer* _sender, const char* stat, gpointer self) {
	_lambda5_ (_sender, stat, self);
}


static void _lambda6_ (Win* obj, Controller* self) {
	g_return_if_fail (obj != NULL);
	controller_seek_forward (self);
}


static void __lambda6__win_seek_forward (Win* _sender, gpointer self) {
	_lambda6_ (_sender, self);
}


static void _lambda7_ (Win* obj, Controller* self) {
	g_return_if_fail (obj != NULL);
	controller_seek_back (self);
}


static void __lambda7__win_seek_back (Win* _sender, gpointer self) {
	_lambda7_ (_sender, self);
}


static void _lambda8_ (Win* obj, Controller* self) {
	g_return_if_fail (obj != NULL);
	controller_volume_up (self);
}


static void __lambda8__win_change_volume_up (Win* _sender, gpointer self) {
	_lambda8_ (_sender, self);
}


static void _lambda9_ (Win* obj, Controller* self) {
	g_return_if_fail (obj != NULL);
	controller_volume_down (self);
}


static void __lambda9__win_change_volume_down (Win* _sender, gpointer self) {
	_lambda9_ (_sender, self);
}


static void _lambda10_ (Win* obj, Controller* self) {
	g_return_if_fail (obj != NULL);
	controller_pause (self);
}


static void __lambda10__win_pause (Win* _sender, gpointer self) {
	_lambda10_ (_sender, self);
}


static void _lambda11_ (Win* obj, Controller* self) {
	g_return_if_fail (obj != NULL);
	controller_quit_running (self);
}


static void __lambda11__win_quit (Win* _sender, gpointer self) {
	_lambda11_ (_sender, self);
}


static void _lambda12_ (Win* obj, Controller* self) {
	g_return_if_fail (obj != NULL);
	controller_next_track (self);
}


static void __lambda12__win_next_track (Win* _sender, gpointer self) {
	_lambda12_ (_sender, self);
}


static void _lambda13_ (Win* obj, Controller* self) {
	g_return_if_fail (obj != NULL);
	controller_previous_track (self);
}


static void __lambda13__win_previous_track (Win* _sender, gpointer self) {
	_lambda13_ (_sender, self);
}


static gboolean _controller_get_player_duration_info_gsource_func (gpointer self) {
	return controller_get_player_duration_info (self);
}


static void controller_start_playing (Controller* self) {
	char* _tmp1_;
	const char* _tmp0_;
	GMainLoop* _tmp2_;
	UriPlayer* _tmp3_;
	Win* _tmp4_;
	g_return_if_fail (self != NULL);
	/*are we playing randomly?*/
	if (controller__random) {
		self->priv->file_array_index = (gint) g_random_int_range ((gint32) 0, (gint32) self->priv->file_array_length1);
	}
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->file_to_open = (_tmp1_ = (_tmp0_ = self->priv->file_array[self->priv->file_array_index], (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->file_to_open = (g_free (self->file_to_open), NULL), _tmp1_);
	/*create the main loop*/
	_tmp2_ = NULL;
	self->priv->loop = (_tmp2_ = g_main_loop_new (NULL, FALSE), (self->priv->loop == NULL) ? NULL : (self->priv->loop = (g_main_loop_unref (self->priv->loop), NULL)), _tmp2_);
	/*create an audio player*/
	_tmp3_ = NULL;
	self->priv->myPlayer = (_tmp3_ = uri_player_new (self->file_to_open), (self->priv->myPlayer == NULL) ? NULL : (self->priv->myPlayer = (g_object_unref (self->priv->myPlayer), NULL)), _tmp3_);
	/*get the initial volume of the player*/
	self->priv->ap_volume = uri_player_get_volume (self->priv->myPlayer);
	g_signal_connect (self->priv->myPlayer, "finished-playing", (GCallback) __lambda1__uri_player_finished_playing, self);
	g_signal_connect (self->priv->myPlayer, "position-duration", (GCallback) __lambda2__uri_player_position_duration, self);
	g_signal_connect (self->priv->myPlayer, "tag-parsed", (GCallback) __lambda3__uri_player_tag_parsed, self);
	g_signal_connect (self->priv->myPlayer, "volume-changed", (GCallback) __lambda4__uri_player_volume_changed, self);
	g_signal_connect (self->priv->myPlayer, "player-status", (GCallback) __lambda5__uri_player_player_status, self);
	/*create a curses ui*/
	_tmp4_ = NULL;
	self->priv->myWin = (_tmp4_ = win_new (), (self->priv->myWin == NULL) ? NULL : (self->priv->myWin = (g_object_unref (self->priv->myWin), NULL)), _tmp4_);
	g_signal_connect (self->priv->myWin, "seek-forward", (GCallback) __lambda6__win_seek_forward, self);
	g_signal_connect (self->priv->myWin, "seek-back", (GCallback) __lambda7__win_seek_back, self);
	g_signal_connect (self->priv->myWin, "change-volume-up", (GCallback) __lambda8__win_change_volume_up, self);
	g_signal_connect (self->priv->myWin, "change-volume-down", (GCallback) __lambda9__win_change_volume_down, self);
	g_signal_connect (self->priv->myWin, "pause", (GCallback) __lambda10__win_pause, self);
	g_signal_connect (self->priv->myWin, "quit", (GCallback) __lambda11__win_quit, self);
	g_signal_connect (self->priv->myWin, "next-track", (GCallback) __lambda12__win_next_track, self);
	g_signal_connect (self->priv->myWin, "previous-track", (GCallback) __lambda13__win_previous_track, self);
	if (!win_run (self->priv->myWin)) {
		return;
	}
	/*we want a function to run every little bit and get data from the player*/
	g_timeout_add ((guint) 300, _controller_get_player_duration_info_gsource_func, self);
	controller_play_audio (self);
	g_main_loop_run (self->priv->loop);
}


static void controller_play_audio (Controller* self) {
	g_return_if_fail (self != NULL);
	uri_player_play (self->priv->myPlayer);
}


static void controller_previous_track (Controller* self) {
	g_return_if_fail (self != NULL);
	/*decrement the index array by two*/
	if (self->priv->file_array_index == 0) {
		self->priv->file_array_index = self->priv->file_array_length1 - 2;
	} else {
		self->priv->file_array_index = self->priv->file_array_index - 2;
	}
	controller_finished_playing (self);
}


static void controller_next_track (Controller* self) {
	g_return_if_fail (self != NULL);
	controller_finished_playing (self);
}


static void controller_finished_playing (Controller* self) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	/*are we playing randomly?*/
	if (controller__random) {
		self->priv->file_array_index = (gint) g_random_int_range ((gint32) 0, (gint32) self->priv->file_array_length1);
	} else {
		self->priv->file_array_index = self->priv->file_array_index + 1;
	}
	/*are we doing a continuous play?*/
	if (self->priv->file_array_index >= self->priv->file_array_length1) {
		if (controller__continuous) {
			self->priv->file_array_index = 0;
		} else {
			/*we are done*/
			controller_quit_running (self);
			return;
		}
	}
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->file_to_open = (_tmp1_ = (_tmp0_ = self->priv->file_array[self->priv->file_array_index], (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->file_to_open = (g_free (self->file_to_open), NULL), _tmp1_);
	uri_player_change_uri (self->priv->myPlayer, self->file_to_open);
	/*we should clear the ncurses screen*/
	win_clear_screen (self->priv->myWin);
	/*play the new track*/
	controller_play_audio (self);
}


static gboolean controller_get_player_duration_info (Controller* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	uri_player_get_duration_info (self->priv->myPlayer);
	return TRUE;
}


static void controller_process_tag (Controller* self, const char* tag, const char* val) {
	GQuark _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (tag != NULL);
	g_return_if_fail (val != NULL);
	_tmp0_ = NULL;
	static GQuark _tmp1__label0 = 0;
	static GQuark _tmp1__label1 = 0;
	static GQuark _tmp1__label2 = 0;
	_tmp0_ = tag;
	_tmp1_ = (NULL == _tmp0_) ? 0 : g_quark_from_string (_tmp0_);
	if (_tmp1_ == ((0 != _tmp1__label0) ? _tmp1__label0 : (_tmp1__label0 = g_quark_from_static_string ("artist"))))
	do {
		win_set_artist (self->priv->myWin, val);
		break;
	} while (0); else if (_tmp1_ == ((0 != _tmp1__label1) ? _tmp1__label1 : (_tmp1__label1 = g_quark_from_static_string ("title"))))
	do {
		win_set_title (self->priv->myWin, val);
		break;
	} while (0); else if (_tmp1_ == ((0 != _tmp1__label2) ? _tmp1__label2 : (_tmp1__label2 = g_quark_from_static_string ("album"))))
	do {
		win_set_album (self->priv->myWin, val);
		break;
	} while (0); else
	do {
		break;
	} while (0);
}


static char* controller_get_human_time (Controller* self, gint64 ns) {
	char* r_val;
	gint64 hours;
	gint64 minutes;
	gint64 sec;
	char* _tmp7_;
	char* _tmp6_;
	g_return_val_if_fail (self != NULL, NULL);
	r_val = g_strdup ("");
	hours = 0LL;
	minutes = 0LL;
	sec = 0LL;
	sec = ns / 1000000000;
	if (sec >= 3600) {
		char* _tmp1_;
		char* _tmp0_;
		char* _tmp3_;
		char* _tmp2_;
		hours = sec / 3600;
		sec = sec - (hours * 3600);
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		r_val = (_tmp1_ = g_strconcat (r_val, _tmp0_ = g_strdup_printf ("%.0f:", (double) hours), NULL), r_val = (g_free (r_val), NULL), _tmp1_);
		_tmp0_ = (g_free (_tmp0_), NULL);
		minutes = sec / 60;
		sec = sec - (minutes * 60);
		_tmp3_ = NULL;
		_tmp2_ = NULL;
		r_val = (_tmp3_ = g_strconcat (r_val, _tmp2_ = g_strdup_printf ("%.0f:", (double) minutes), NULL), r_val = (g_free (r_val), NULL), _tmp3_);
		_tmp2_ = (g_free (_tmp2_), NULL);
	} else {
		if (sec >= 60) {
			char* _tmp5_;
			char* _tmp4_;
			minutes = sec / 60;
			sec = sec - (minutes * 60);
			_tmp5_ = NULL;
			_tmp4_ = NULL;
			r_val = (_tmp5_ = g_strconcat (r_val, _tmp4_ = g_strdup_printf ("%.0f:", (double) minutes), NULL), r_val = (g_free (r_val), NULL), _tmp5_);
			_tmp4_ = (g_free (_tmp4_), NULL);
		}
	}
	_tmp7_ = NULL;
	_tmp6_ = NULL;
	r_val = (_tmp7_ = g_strconcat (r_val, _tmp6_ = g_strdup_printf ("%.0f", (double) sec), NULL), r_val = (g_free (r_val), NULL), _tmp7_);
	_tmp6_ = (g_free (_tmp6_), NULL);
	return r_val;
}


static void controller_update_status (Controller* self) {
	gint vol;
	char* str;
	g_return_if_fail (self != NULL);
	vol = (gint) (self->priv->ap_volume * 100);
	str = g_strdup_printf ("(%s/%s) %i%% Volume:%i %s", self->priv->ap_play_time, self->priv->ap_total_time, self->priv->ap_percent_played, vol, self->priv->ap_state);
	win_set_status (self->priv->myWin, str);
	str = (g_free (str), NULL);
}


static void controller_process_position_duration (Controller* self, gint64 position, gint64 duration) {
	float percent;
	g_return_if_fail (self != NULL);
	percent = 0.0F;
	if (duration < 1) {
	} else {
		char* _tmp0_;
		char* _tmp1_;
		/*there is a problem with the data*/
		_tmp0_ = NULL;
		self->priv->ap_play_time = (_tmp0_ = controller_get_human_time (self, position), self->priv->ap_play_time = (g_free (self->priv->ap_play_time), NULL), _tmp0_);
		_tmp1_ = NULL;
		self->priv->ap_total_time = (_tmp1_ = controller_get_human_time (self, duration), self->priv->ap_total_time = (g_free (self->priv->ap_total_time), NULL), _tmp1_);
		percent = (float) ((10000 * position) / duration);
		percent = percent / 100;
		self->priv->ap_percent_played = (gint) percent;
		win_set_played_percent (self->priv->myWin, self->priv->ap_percent_played);
	}
	controller_update_status (self);
}


/*we need a way to quit the mainloop*/
static void controller_quit_running (Controller* self) {
	g_return_if_fail (self != NULL);
	/*stop the loop*/
	g_main_loop_quit (self->priv->loop);
}


/*we need to have a "help" that we need to print for the user*/
static void controller_print_help (Controller* self, const char* message) {
	char* help;
	char* _tmp3_;
	char* _tmp2_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (message != NULL);
	help = g_strdup ("");
	if (string_get_length (message) > 0) {
		char* _tmp1_;
		const char* _tmp0_;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		help = (_tmp1_ = (_tmp0_ = message, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), help = (g_free (help), NULL), _tmp1_);
	}
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	help = (_tmp3_ = g_strconcat (help, _tmp2_ = g_option_context_get_help (self->priv->optContext, TRUE, NULL), NULL), help = (g_free (help), NULL), _tmp3_);
	_tmp2_ = (g_free (_tmp2_), NULL);
	fprintf (stdout, help);
	help = (g_free (help), NULL);
}


/*parse signals from the UI*/
static void controller_seek_forward (Controller* self) {
	g_return_if_fail (self != NULL);
	uri_player_seek_forward (self->priv->myPlayer, 5);
}


static void controller_seek_back (Controller* self) {
	g_return_if_fail (self != NULL);
	uri_player_seek_back (self->priv->myPlayer, 5);
}


static void controller_volume_up (Controller* self) {
	g_return_if_fail (self != NULL);
	uri_player_increase_volume (self->priv->myPlayer, 0.03);
}


static void controller_volume_down (Controller* self) {
	g_return_if_fail (self != NULL);
	uri_player_decrease_volume (self->priv->myPlayer, 0.03);
}


static void controller_pause (Controller* self) {
	g_return_if_fail (self != NULL);
	uri_player_pause (self->priv->myPlayer);
}


static void controller_class_init (ControllerClass * klass) {
	controller_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ControllerPrivate));
	G_OBJECT_CLASS (klass)->finalize = controller_finalize;
}


static void controller_instance_init (Controller * self) {
	self->priv = CONTROLLER_GET_PRIVATE (self);
}


static void controller_finalize (GObject* obj) {
	Controller * self;
	self = CONTROLLER (obj);
	(self->priv->myPlayer == NULL) ? NULL : (self->priv->myPlayer = (g_object_unref (self->priv->myPlayer), NULL));
	(self->priv->myWin == NULL) ? NULL : (self->priv->myWin = (g_object_unref (self->priv->myWin), NULL));
	self->priv->file_array = (_vala_array_free (self->priv->file_array, self->priv->file_array_length1, (GDestroyNotify) g_free), NULL);
	self->priv->version = (g_free (self->priv->version), NULL);
	self->priv->summary = (g_free (self->priv->summary), NULL);
	self->priv->description = (g_free (self->priv->description), NULL);
	self->priv->status_text = (g_free (self->priv->status_text), NULL);
	self->file_to_open = (g_free (self->file_to_open), NULL);
	(self->priv->loop == NULL) ? NULL : (self->priv->loop = (g_main_loop_unref (self->priv->loop), NULL));
	self->priv->ap_play_time = (g_free (self->priv->ap_play_time), NULL);
	self->priv->ap_total_time = (g_free (self->priv->ap_total_time), NULL);
	self->priv->ap_state = (g_free (self->priv->ap_state), NULL);
	(self->priv->web_regex == NULL) ? NULL : (self->priv->web_regex = (g_regex_unref (self->priv->web_regex), NULL));
	(self->priv->optContext == NULL) ? NULL : (self->priv->optContext = (g_option_context_free (self->priv->optContext), NULL));
	G_OBJECT_CLASS (controller_parent_class)->finalize (obj);
}


GType controller_get_type (void) {
	static GType controller_type_id = 0;
	if (controller_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ControllerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) controller_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Controller), 0, (GInstanceInitFunc) controller_instance_init, NULL };
		controller_type_id = g_type_register_static (G_TYPE_OBJECT, "Controller", &g_define_type_info, 0);
	}
	return controller_type_id;
}


static char** _vala_array_dup1 (char** self, int length) {
	char** result;
	int i;
	const char* _tmp0_;
	result = g_new0 (char*, length);
	for (i = 0; i < length; i++) {
		result[i] = (_tmp0_ = self[i], (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	}
	return result;
}


/*this is the function that starts everything*/
void _main (char** args, int args_length1) {
	GError * _inner_error_;
	char** _tmp2_;
	gint input_size;
	gint input_length1;
	char** _tmp1_;
	const char* _tmp0_;
	char** input;
	Controller* myController;
	_inner_error_ = NULL;
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	input = (_tmp2_ = (_tmp1_ = args, (_tmp1_ == NULL) ? ((gpointer) _tmp1_) : _vala_array_dup1 (_tmp1_, args_length1)), input_length1 = args_length1, input_size = input_length1, _tmp2_);
	/*make sure we have thread support*/
	if (!g_thread_supported ()) {
		fprintf (stderr, "Cannot run without threads.\n");
		input = (_vala_array_free (input, input_length1, (GDestroyNotify) g_free), NULL);
		return;
	}
	/* Initializing GStreamer*/
	gst_init (&args_length1, &args);
	myController = controller_new ();
	{
		controller_ParseArgs (myController, &input, &input_length1, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_OPTION_ERROR) {
				goto __catch2_g_option_error;
			}
			goto __finally2;
		}
	}
	goto __finally2;
	__catch2_g_option_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			fprintf (stdout, "%s\n", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			input = (_vala_array_free (input, input_length1, (GDestroyNotify) g_free), NULL);
			(myController == NULL) ? NULL : (myController = (g_object_unref (myController), NULL));
			return;
		}
	}
	__finally2:
	if (_inner_error_ != NULL) {
		input = (_vala_array_free (input, input_length1, (GDestroyNotify) g_free), NULL);
		(myController == NULL) ? NULL : (myController = (g_object_unref (myController), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	/* run the controller*/
	controller_run (myController);
	input = (_vala_array_free (input, input_length1, (GDestroyNotify) g_free), NULL);
	(myController == NULL) ? NULL : (myController = (g_object_unref (myController), NULL));
}


int main (int argc, char ** argv) {
	g_thread_init (NULL);
	g_type_init ();
	_main (argv, argc);
	return 0;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}




